import falcon


class ContentTypeManager(object):
    """Class for content type Manager"""

    def process_request(self, req, resp):
        """process_request is a middleware for
        checking the application/json request.
        Args:
            req (object): request object
            resp (object): response object

        Status:
            Stable

        Raises:
            HTTPBadRequest:
                request object does not have a application/json
                in req.content_type
        """
        if req.content_type is None or "application/json" not in req.content_type:
            raise falcon.HTTPBadRequest(
                title="Invalid Request",
                description="All requests must have a application/json header",
            )
        else:
            return



import falcon


class ServerMaintainceManager(object):

    """
        Server maintance manager, for checking if the server is under maintance.
        Useage:
            >>> from camembert.middlewares import ServerMaintainceManager
            >>> api = falcon.api(middlware=[ServerMaintainceManager(True)])
        
        You can configure the bool value to come from anywhere
        1. A configuration file.
        2. DB 
        3. or hard code it. :(
    """

    def __init__(self, server_state):

        """
            Initialization method

            Args:
                server_state (bool): True if server is under maintance.

            Raises:
                falcon.HTTPInvalidParam
            
            Returns:
                None
        """
        if server_state is None:
            raise falcon.HTTPInvalidParam(
                title="Invalid Param", description="Server State should be passed."
            )
        if type(server_state) is not bool:
            raise falcon.HTTPInvalidParam(
                title="Invalid Param",
                description="Invalid datatype. Server Maintaince manager accepts bool.",
            )
        else:
            self.server_state = server_state

    def process_request(self, req, resp):
        """
            process_request, processing the incoming request.

            Args:
                req (object)
                resp (object)
            
            Returns: 
                None
            
            Raises:
                falcon.HTTPInvalidParam
        """
        if self.server_state is True:
            raise falcon.HTTPServiceUnavailable(
                title="Service unavailable.",
                description="Service is currently unavailable. Please try again later.",
            )
        else:
            return



import falcon

class SuccessResponseManager(object):
    """
       A manager middleware for changing the security headers in success response.
    """

    def __init__(self, response_headers):
        """
            Initialization method for SucessResponseManager

            Args:
                response_headers(object): Response headers for setting response header.

            Returns:
                None

            Raises:
                falcon.HTTPBadRequest
        """
        if isinstance(response_headers, dict):
            self.headers = response_headers
        else:
            raise falcon.HTTPBadRequest(
                title="Bad Request",
                description="Response headers should be a dictonary.",
            )

    def process_response(self, req, resp, resource, req_succeeded):
        """
        process_response - It can process successful responses and
        set headers to every header.
        This middlware does not deletes headers which are vulnerable.
        Please use use another middleware in deleting unwanted response headers.
        Please refer https://www.owasp.org for more details.

        Args:
            req (object): request object
            resp (object): response object
            resporce (object): Target respource
            req_succeeded (bool): Does this response instance succedded

        Raises:
            None

        Status:
            Stable

        Returns:
            None
        """
        for key, value in self.headers.items():
            resp.set_header(key, value)
        return



import falcon
import json


class JSONTranslatorManager(object):
    """JSONTranslatorManager for translating JSON"""

    def process_request(self, req, resp):
        """
            process_request, translates json

            Args:
                req (object): request object
                resp (object): response object

            Returns:
                object: sets data as a req param

            Status:
                Stable
                
            Raises:
                falconHTTPBadRequest
        """
        if req.method == "GET":
            return
        else:
            try:
                if req.content_length:
                    req.data = json.loads(req.bounded_stream.read().decode("utf-8"))
                    return
                else:
                    raise falcon.HTTPBadRequest(
                        "Bad request", "Cannot accept content lenght null."
                    )
            except Exception:
                raise falcon.HTTPBadRequest(
                    title="Bad request",
                    description="Invalid body. \
                        Unable to parse the given content",
                )



"""Module for maintaing logs for server."""
import os
import logging
import uuid
from datetime import datetime


def setup_logging(log):
    """
        setup_logging - A server logging setting up resource.

        Args:
            log(object): A log object from process_request

        Returns:
            None

        Raises:
            None
    """
    file_path = os.getcwd()
    log_location = file_path + "/logs/"
    if not os.path.exists(log_location):
        os.makedirs(log_location)
    current_time = datetime.now()
    current_date = current_time.strftime("%Y-%m-%d")
    file_name = current_date + ".log"
    file_location = log_location + file_name

    logging.basicConfig(filename=file_location, level=logging.DEBUG)
    with open(file_location, "a+"):
        pass

    logger = logging.getLogger(__name__)
    logger.info(log)
    return None


class ServerLoggingManager:
    """
        This module is responsible for maintaing
        logs in a .log file. These are the currently
        implemented details for each log entry.
        1. UUID for each request.
        2. datetime
        3. Method
        4. URI
        5. Status
    """

    def process_response(self, req, resp, resource, req_succeeded):
        """
            process_response - Process response from Falcon server.

        Args:
            req (object): Request object
            resp (object): Response object
            resource (object): Resources targetted.
            req_succeeded (bool): True if succeded

        Returns:
            None

        Raises:
            None
        """
        construct_logger_info = "{0} {1} {2} {3} {4}".format(
            uuid.uuid4(), datetime.now(), req.method, req.uri, resp.status
        )
        setup_logging(construct_logger_info)
        return



"""
    Middleware for limiting user access to the server globally.
    Inspired from https://pypi.org/project/ratelimiter/
"""
import collections
import time
import falcon


class RateLimiterManager(object):
    """
        A middlware manager for limiting API calls.
        It raises a falcon error if hit with too much requests.

    """

    def __init__(self, max_calls, time_period=1.0):
        """
            Initialization method for RateLimiterManager.

            Args:
                max_calls (int): Maximum number of calls for the server.
                time_period (float): Time limit for the API. This time is set to 1.0.

            Returns:
                None

            Status:
                Expiremental

            Raises:
                falcon.HTTPPreconditionFailed
        """
        if time_period <= 0:
            raise falcon.HTTPPreconditionFailed(
                title="Server HTTP Pre-Condition Failed",
                description="Time period cannot be less than 0.",
            )
        if max_calls <= 0:
            raise falcon.HTTPPreconditionFailed(
                title="Server HTTP Pre-Condition Failed",
                description="Maximum number of calls should be more than 0.",
            )
        self.calls = collections.deque()
        self.max_calls = max_calls
        self.time_period = time_period
        self._timespan = 0

    def process_request(self, req, resp):
        """
            process_request - Processing incoming falcon request.

            Args:
                req (object): Request object
                resp (object): Response object

            Returns:
                None

            Status:
                Expiremental

            Raises:
                falcon.HTTPLocked: Falcon HTTP error informing user of resource is been locked.
                falcon.HTTPPreconditionFailed: Falcon HTTP error is raised when the user is
                    passing values less than the threshold time or response limit.
        """
        self.calls.append(time.time())
        if len(self.calls) > self.max_calls:
            raise falcon.HTTPLocked(
                title="Resource locked.",
                description="The resource you requested is currently locked.",
            )
        else:
            return

    def process_response(self, req, resp, resource, req_succeeded):
        """
            process_response - Process response from falcon.

            Args:
                req (object): Request object.
                resp (object): Response object
                Resource (object): Resource object
                req_succeeded (bool): is request succeeded or not.

            Status:
                Expiremental

            Returns:
                None

            Raises:
                None
        """
        self._timespan = self.calls[-1] - self.calls[0]
        while self._timespan >= self.time_period:
            if len(self.calls) == 0:
                break
            else:
                self.calls.popleft()
        return



import re
import falcon

_FORWARDED_PROTO_RE = re.compile("proto=([A-Za-z]+)")


class RequireHTTPSManager(object):
    """Middleware to verify that each request is performed via HTTPS.

    While the web server is primarily responsibile for enforcing the
    HTTPS protocol, misconfiguration is still a leading cause of
    security vulnerabilities, and so it can be helpful to perform
    certain additional checks, such as this one, within the application
    layer itself.

    At least one of the following sources must indicate the use of
    HTTPS:

        * The requested URI
        * The X-Forwarded-Proto header
        * The Forwarded header

    Otherwise, an instance of falcon.HTTPBadRequest is raised.
    """

    def process_request(self, req, resp):
        # NOTE: replaced deprecated req.protocol
        #       with req.scheme since it removed in Falcon v2.0.
        if req.scheme.lower() == "https":
            return

        xfp = req.get_header("X-FORWARDED-PROTO")
        if xfp and xfp.lower() == "https":
            return

        forwarded = req.get_header("FORWARDED")
        if forwarded:
            # NOTE(kgriffs): The first hop must indicate HTTPS,
            #   otherwise the chain is already insecure.
            first, __, __ = forwarded.partition(",")

            match = _FORWARDED_PROTO_RE.search(first)
            if match and match.group(1).lower() == "https":
                return

        raise falcon.HTTPBadRequest(
            title="HTTPS Required",
            description=(
                "All requests must be performed via the HTTPS protocol. Please switch to HTTPS and try again."
            ),
        )



import sys
from setuptools import setup

__version__ = list(map(str, [0, 11, 0]))

if sys.version_info < (3, 0):
    raise Exception("You require Python3.0 or above to run Camembert")

with open("README.md") as f:
    long_description = f.read()

setup(
    name="camembert",
    version=".".join(__version__),
    description="Falcon middlewares made easy.",
    url="https://github.com/Jithinqw/Camembert",
    author="Jithin Zacharia",
    author_email="zachariajithin@gmail.com",
    license="MIT",
    packages=["camembert"],
    long_description=long_description,
    long_description_content_type="text/markdown",
    download_url="https://github.com/Jithinqw/Camembert/archive/0.11.0.tar.gz",
    install_packages=["falcon"],
    install_requires=["falcon"],
    keywords=["falcon", "middlewares"],
)



"""
Middleware for managing internal server errors,
and response with a apt error message
"""
import falcon


class InternalServerErrorManager(object):
    """Middleware for managing internal server errors"""

    def process_response(self, request, resp, resource, req_succeeded):
        """
            Manages response if the server encounters any internal server errors.
            For 500 status

            Args:
                req (object): request object
                resp (object): response object
                resource (object): Target respource
                req_succeeded (bool): Does this response succedded

            Returns:
                None

            Raises:
                falcon.HTTPInternalServerError: Raises Falcon internal server error.
        """
        if resp.status == falcon.HTTP_500:
            raise falcon.HTTPInternalServerError(
                title="Internal Server Error",
                description="Something went wrong on our side. Please try again later.",
            )
        else:
            return



import falcon


class BlacklistedIpManager(object):
    """
        Middleware for managing blacklisted ip addresss
        for a server.

        Usage:

        >>> from camembert.middlewares import BlacklistedIpManager
        >>> app = falcon.api(
            middlewares=[
                BlacklistedIpManager(IPAddress_list)
            ]
        )

        A list of ip address string must be passed to the middleware.
    """

    def __init__(self, black_ip_address):
        if black_ip_address is None:
            raise falcon.HTTPPreconditionFailed(
                title="Server precondition failed",
                description="list of black listed IPs cannot be empty.",
            )
        if isinstance(black_ip_address, list) is not True:
            raise falcon.HTTPBadRequest(
                title="Bad Request",
                description="Black listed IP must be passed as a list.",
            )
        else:
            self.black_ip = black_ip_address

    def process_request(self, req, resp):
        """
            process_request

            Args:
                req (object)
                resp (object)

            Returns:
                None

            Raises:
                falcon.HTTPUnauthorized
        """
        if req.remote_addr in self.black_ip:
            raise falcon.HTTPUnauthorized(
                title="Unauthorized",
                description="You are unauthorized to view this route.",
            )
        else:
            return
